#
# NeoScript - Server Side Programming based on Safe Tcl
#
# Copyright (C) NeoSoft, All Rights Reserved.
#
# $Id: common.tcl,v 1.10 1996/06/26 23:04:39 kunkee Exp $
#

#
# PROPERTY LIST STORAGE AND RETRIEVAL
#
set parallelBase "$server(SERVER_ROOT)/neoscript-data/users"

proc setup_data_access {} {
    global parallelDir webpageOwnerName webenv

    if [info exists parallelDir] return

    global webenv parallelBase

    set webpageOwnerName [id convert userid $webenv(NEO_DOCUMENT_UID)]
    set parallelDir $parallelBase/[cindex $webpageOwnerName 0]/$webpageOwnerName/
}

proc db_name_check {dbname dbtype fileNameVar} {
    if ![regexp {^[a-zA-Z0-9_]*$} $dbname] {
	error "illegal database name: $dbname, upper/lowercase and 0-9 only"
    }
    upvar $fileNameVar dbFileName
    global parallelDir

    set dbFileName ${parallelDir}$dbtype/$dbname.$dbtype
}

proc create_user_dir {plusSubDir} {
    global parallelDir errorCode

    setup_data_access

    if {[catch {mkdir -path $parallelDir$plusSubDir} result] == 1} {
	if {[lrange $errorCode 0 1] != "POSIX EEXIST"} {
	    error "$errorCode"
	}
    }
}

proc dbdelkey {database id} {
    global parallelDir errorCode

    setup_data_access

    db_name_check $database db dbFileName

    if {[catch {set db [db open $dbFileName hash rw 0664]}] == 1} {
	if {[lrange $errorCode 0 1] != "POSIX ENOENT"} {
	    error "$dbFileName: $errorCode"
	} else {
	    return 0
	}
    }
    set result [db del $db $id]
    db close $db
    return $result
}

proc dbfetch {database id arrayName} {
    global parallelDir errorCode

    upvar $arrayName array

    setup_data_access

    db_name_check $database db dbFileName

    if {[catch {db open $dbFileName hash rwl 0664} db] == 1} {
	if {[lrange $errorCode 0 1] != "POSIX EEXIST"} {
	   error $errorCode
	}
	return 0
    }
    db get $db $id list
    db close $db

    set haveData [expr ![lempty $list]]
    array set array $list
    return $haveData
}

proc dbstore {database id arrayName} {
    global parallelDir errorCode

    upvar $arrayName array

    setup_data_access

    db_name_check $database db dbFileName

    set list [array get array]

    if {[catch {set db [db open $dbFileName hash rwL 0664]}] == 1} {
	if {[lrange $errorCode 0 1] != "POSIX ENOENT"} {
	    error "$dbFileName: $errorCode"
	} else {
            create_user_dir db
	    set db [db open $dbFileName hash ctL 0664]
	}
    }
    db put $db $id $list
    db close $db
    return
}

proc dbexists {database} {
    global parallelDir

    setup_data_access
    db_name_check $database db dbFileName

    return [file readable $dbFileName]
}

proc dbkeys {database {pattern *}} {
    global errorCode parallelDir

    setup_data_access

    db_name_check $database db dbFileName

    if {[catch {set db [db open $dbFileName hash rw 0664]}] == 1} {
	if {[lrange $errorCode 0 1] != "POSIX ENOENT"} {
	    error "$dbFileName: $errorCode"
	} else {
	    return ""
	}
    }
    set keys ""
    db searchall $db key -glob $pattern {
	lappend keys $key
    }
    db close $db

    return $keys
}

proc dbdump {database {pattern *}} {
    global parallelDir

    setup_data_access

    db_name_check $database db dbFileName

    set db [db open $dbFileName hash rw 0664]
    set keys ""
    db searchall $db key -glob $pattern {
	html "\[$key\]\n"
	db get $db $key list
	foreach pair $list {
	    html "[lindex $pair 0]=[lindex $pair 1]\n"
	}
	html "\n"
    }
    db close $db

    return
}

proc dbfiles {} {
    global parallelDir errorCode
    setup_data_access

    foreach file [glob $parallelDir/db/*.db] {
	lappend result [file root [file tail $file]]
    }
    return $result
}

# proc to import key value pairs of the form "-color blue"
# into an array in the caller's context.
#
# and dialog box thingie that uses it
#
# i am really missing incr tcl
#
#
#neo_dialog -text "This is an alert mesesage... or something."
#
#neo_dialog -text "This is another alert message."
#

proc import_keyvalue_pairs {arrayName string} {
    upvar $arrayName array

    set len [llength $string]
    if {$len % 2 != 0} {
        error "unmatched key-value pair"
    }

    for {set i 0} {$i < $len} {incr i 2} {
        set key [lindex $string $i]
        if {[string index $key 0] != "-"} {
            error "key $key of key-value pairs doesn't start with a dash"
        }
        set array([string range $key 1 end]) [lindex $string [expr $i + 1]]
    }
}

#
# List the data files in the server-maintained data directory
# for the user.
#
#
proc list_data_files {} {
    global parallelDir errorCode
    setup_data_access

    foreach file [glob $parallelDir/data/*.data] {
	lappend result [file root [file tail $file]]
    }
    return $result
}

