#
# NeoScript [TM] development source code
#
# Tcl file that, when debugging is true, gets sourced every time a webpage 
# is pulled up that contains Neoscript server-side code. 
#
# If debugging is false, the file is loaded in only once by the
# "mother thread".
#
# $Id: devel.tcl,v 1.41 1996/06/28 20:30:56 kunkee Exp $
#

proc dump_request_stack {} {
    if ![www_request_info req] {
        error "impossible?  www_request_info failed for current record"
    }
    html "Dump of request stack" h2
    html "Cazart!  Something went wrong and you get to see all this cool stuff!" b
    parray req

    foreach type "next prev main" {
        if {$req($type)} {
            for {set i 1} {[www_request_info $type $i nreq]} {incr i} { 
                html "$type $i" h2
                parray nreq
            }
        }
    }
}
 
proc SAFE_extend_client {safeInterp} {
    global clientLoadData

    if ![info exists clientLoadData] {
	global server

	set fp [open $server(SERVER_ROOT)/conf/client.tcl]
	set clientLoadData [read $fp]
	close $fp
    }
    $safeInterp eval $clientLoadData
}

#
#
# "database" is inoperative -- currently uses only the tcl_archive wais
# database.
#
proc perform_wais_query {database searchString} {
    set fp [open "|/usr/local/bin/waisq -f - -m 20 -S /htdocs/tmp/tcl_archive.src -g $searchString"]
    set result ""
    while {[gets $fp line] >= 0} {
	set line [string trim $line]
	if {[string match :score* $line]} {
	    set score [lindex $line 1]
	    if {$score == "0"} {
		return ""
	    }
	}
	if {[string match :headline* $line]} {
	    lappend result [list $score [crange $line 11 end-1]]
	}
    }
    return $result
}

#
# run the special code to setup whatever is loaded by the
# devel.tcl file.  It is expected that code in here that
# works will be migrated to init.tcl if it is found to be useful.
#
proc devel_setup {safeInterp} {
    $safeInterp alias load_extensions SAFE_extend_client $safeInterp
    SAFE_extend_client $safeInterp
    $safeInterp alias perform_wais_query perform_wais_query

    $safeInterp alias using_otcl using_otcl $safeInterp
    $safeInterp alias rootname file rootname

    global NeoWebServerConf
    if {[info exists NeoWebServerConf(HavePostgres)] \
	&& $NeoWebServerConf(HavePostgres)} {
	    postgres_setup $safeInterp
    }
}

proc postgres_setup {safeInterp} {
    global NeoWebDirConf
    load {} Pg $safeInterp
    if {![info exists NeoWebDirConf(PostgresAny)] || $NeoWebDirConf(PostgresAny) == 0} {
	$safeInterp alias pg_connect SAFE_pg_connect $safeInterp
	$safeInterp alias pg_createdb SAFE_pg_createdb
    }
}

proc SAFE_pg_connect {safeInterp {databaseName {}} args} {
    global webpageOwnerName

    setup_data_access
    if {[lempty $databaseName] || $databaseName == $webpageOwnerName} {
      set databaseName $webpageOwnerName
    } else {
      set databaseName ${webpageOwnerName}__${databaseName}
    }
    set pgp_handle [eval pg_connect $databaseName $args]
    return [pg_handle $safeInterp $pgp_handle]
}

proc SAFE_pg_createdb {{databaseName {}}} {
    global webpageOwnerName

    setup_data_access
    if {[lempty $databaseName] || $databaseName == $webpageOwnerName} {
      set databaseName $webpageOwnerName
    } else {
      set databaseName ${webpageOwnerName}__${databaseName}
    }
    
    set sql_command "create database $databaseName;"
    set postgresFP [pg_connect template1]
    set result [pg_exec $postgresFP $sql_command]
    pg_disconnect $postgresFP
    pg_result $result -clear
    return
}

proc SAFE_du {} {
    global webpageOwnerName
    setup_data_access

    set first [cindex $webpageOwnerName 0]
    set path "/usr/ftp/pub/users/$first/$webpageOwnerName"
    set duFP [open "|/usr/bin/du $path" r]
 
    while {[gets $duFP line] > 0} {
        html $line
        html "<br>"
    }
    close $duFP
}


# assumes that Postgres is already loaded
proc using_otcl {safeInterp} {
    global server
    load {} otcl $safeInterp
    $safeInterp eval {proc debug args { }; set in_emulator 0}
    $safeInterp eval [read_file $server(SERVER_ROOT)/conf/class.tcl]
}
